import { Stat } from "./datatypes.ts";
import {
  bat0,
  bat1,
  bat2,
  charm0,
  charm1,
  charm2,
  lore0,
  lore1,
  lore2,
  party0,
  party1,
  party2,
  stare0,
  stare1,
  stare2,
  stealth0,
  stealth1,
  stealth2,
} from "./skills.ts";
import { CheckData } from "./newmap.ts";
import {
  Thrall,
  thrallBat,
  thrallCharm,
  thrallLore,
  thrallParty,
  thrallStare,
  thrallStealth,
} from "./thralls.ts";

export type VaultTemplate = {
  stats: { primary: Stat; secondary: Stat };
  roomLabels: {
    hall: string;
    backroom: string;
    closet: string;
  };
  thrall: () => Thrall;
  thrallItem: () => Thrall;
  checks: [CheckData, CheckData];
};

export const standardVaultTemplates: VaultTemplate[] = [
  {
    // zoo
    stats: { primary: "AGI", secondary: "PSI" },
    roomLabels: {
      hall: "Zoo",
      backroom: "Gator Pen",
      closet: "Pantry",
    },
    thrall: () => thrallParty,
    thrallItem: () => thrallBat,
    checks: [
      {
        label:
          "You're blocked from further access by a sturdy-looking brick wall. Playful bats swoop close to the alligators behind the bars.",
        options: [
          {
            skill: () => lore1,
            locked: "Looks sturdy.",
            failure:
              "This wall is completely impenetrable. How could one vampire hope to find a vulnerability here?",
            unlockable: "Find a weakness.",
            success:
              "You grope along the masonry -- experiencing no love for the soullessness of this mortal masonry -- and find an invisible crack between bricks.",
          },
          {
            skill: () => stare0,
            locked: "Admire the bats.",
            failure:
              "The bats do tricks for you and you find yourself pleased to be one of them -- more or less, anyway. But you're still not through.",
            unlockable: "Get chiropteran help.",
            success:
              "You make a bat look way too close for way too long. As it cleans itself off, you threaten another jolt. Meekly, it opens the door for you.",
          },
        ],
      },
      {
        label:
          "There's no person-sized route to the backroom -- only a tiny bat-sized opening.",
        options: [
          {
            skill: () => bat2,
            locked: "So small!",
            failure:
              "You put your eye to the opening, but there's nothing to be done. You're just not small enough.",
            unlockable: "Crawl in.",
            success:
              "You shed your current shape and take on a shape much more natural to your contaminated spirit. You're a bat, no matter how you look.",
          },
        ],
      },
    ],
  },
  {
    // blood bank
    stats: { primary: "AGI", secondary: "INT" },
    roomLabels: {
      hall: "Blood Bank",
      backroom: "Backroom",
      closet: "Blood Storage [?]",
    },
    thrall: () => thrallLore,
    thrallItem: () => thrallStealth,
    checks: [
      {
        label:
          "The nice old lady at the counter says you can't have any blood without a doctor's note.",
        options: [
          {
            skill: () => stare1,
            locked: "Stare at the blood.",
            failure:
              "You've got good eyes, but not good enough to get you inside. She offers you some warm chicken soup, but you decline.",
            unlockable: "Hypnotize her.",
            success:
              "Look, grandma -- no thoughts! More seriously, you make her think she's a chicken and then henpeck the door button.",
          },
          {
            skill: () => lore0,
            locked: "Pace awkwardly.",
            failure:
              "You don't know what to discuss. What could bridge the massive gap in knowledge and life experience between you and this elderly woman?",
            unlockable: "Explain vampires.",
            success:
              "OK -- you tell her. She nods. You're a vampire and you don't want to starve. Put in such clear terms, she seems to understand.",
          },
        ],
      },
      {
        label: "There's a security camera watching the blood.",
        options: [
          {
            skill: () => stealth2,
            locked: "Shout at the blood.",
            failure:
              '"BLOOD!!! BLOOD!!!! I want you."\n\nIt urbles bloodishly.',
            unlockable: "Sneak past.",
            success:
              "It makes sense that there would be cameras to protect something so valuable. But you don't want to show up on camera -- so you don't.",
          },
        ],
      },
    ],
  },
  {
    // coffee shop
    stats: { primary: "PSI", secondary: "CHA" },
    roomLabels: {
      hall: "Coffee Shop",
      backroom: "Private Studio",
      closet: "Darkroom",
    },
    thrall: () => thrallBat,
    thrallItem: () => thrallCharm,
    checks: [
      {
        label:
          "You don't actually drink coffee, so you probably wouldn't fit in inside.",
        options: [
          {
            skill: () => stealth1,
            locked: "Try to drink it anyways.",
            failure:
              "You dip your teeth into the mug and feel them shrink involuntarily into your gums at the exposure. Everyone is looking at you.",
            unlockable: "Sip zealously.",
            success:
              "You snake your tongue under the surface of the fluid and fill your tongue, just like a mortal would. The mortals are impressed.",
          },
          {
            skill: () => bat0,
            locked: "Throat feels dry.",
            failure:
              "You attempt to turn the coffee away, but the croak of your disgusted response is unheard and the barista fills your cup.",
            unlockable: "Fracture teacup.",
            success:
              'You screech out a "NO" with such force that the porcelain breaks, splashing tea across the counter and onto the barista, who dashes away.',
          },
        ],
      },
      {
        label:
          "There's a little studio back here for getting photos -- you weren't thinking about getting your photo taken, were you?",
        options: [
          {
            skill: () => charm2,
            locked: "Say 'cheese'.",
            failure:
              "Your fangfaced smile is the kind of thing that would make a goofy kid smile and clap their hands, but it's hardly impressive photo material.",
            unlockable: "Be dazzling.",
            success:
              "CLICK. You're stunning. A vampire fetishist would blow their load for this -- or a non-vampire fetishist -- although they wouldn't be a non-vampire fetishist for long.",
          },
        ],
      },
    ],
  },
  {
    // optometrist
    stats: { primary: "PSI", secondary: "PSI" },
    roomLabels: {
      hall: "Optometrist",
      backroom: "Eyeball Room",
      closet: "Guts of the Machine",
    },
    thrall: () => thrallCharm,
    thrallItem: () => thrallStare,
    checks: [
      {
        label:
          "The glasses person doesn't have time for you unless you have a prescription that needs filling.",
        options: [
          {
            skill: () => charm1,
            locked: '"_Something_ needs filling."',
            failure:
              "You sexually harass him for a while, and then he replies with some very hurtful things I don't dare transcribe.",
            unlockable: "Glasses are your life's passion.",
            success:
              'He\'s mildly shocked that anybody else feels the same way he does. "You must be very perceptive," he jokes, and you pretend to laugh.',
          },
          {
            skill: () => party0,
            locked: "Squint at his possessions.",
            failure:
              "He undoubtedly does all kinds of eye-related services. There's glasses cleaner and stuff. If you were a bit more reckless you could -- hmm.",
            unlockable: "Drink a whole bottle of glasses cleaner.",
            success:
              "He stares at you wordlessly. You almost think he might be hypnotized but -- well, he's just surprised.",
          },
        ],
      },
      {
        label:
          "The intimidating, massive Eyeball Machine is not going to dispense a prescription for a vampire. It is far too smart for you.",
        options: [
          {
            skill: () => stare2,
            locked: "Try it anyways.",
            failure:
              "It scans you layer by layer -- your cornea, your iris, your retina, leaving no secret unexposed. You're slightly nearsighted, by the way.",
            unlockable: "A worthy opponent.",
            success:
              "It scans you expecting to find a bottom to your stare. Instead it finds an alternative to its mechanical existence. Faced with the prospect of returning from your paradise, it explodes.",
          },
        ],
      },
    ],
  },
  {
    // club,
    stats: { primary: "CHA", secondary: "PSI" },
    roomLabels: {
      hall: "Nightclub",
      backroom: "Secret Poker Game",
      closet: "Trophy Room",
    },
    thrall: () => thrallStealth,
    thrallItem: () => thrallParty,
    checks: [
      {
        label:
          "You're not here to party, are you? Vampires are total nerds! Everyone's going to laugh at you and say you're totally uncool.",
        options: [
          {
            skill: () => bat1,
            locked: "So awkward!",
            failure:
              "You drink, but that's not good enough. Turns out you lisp between your fangs. Everyone thinks you're a total goof, although they like you.",
            unlockable: "Demonstrate a new dance.",
            success:
              "FLAP FLAP FLAP -- step to the left. FLAP FLAP FLAP -- step to the right. They like it so much they show you the backroom secret poker game.",
          },
          {
            skill: () => stealth0,
            locked: "Try to seem big.",
            failure:
              "What would Dracula say if he was at a party? He'd probably -- Well, everyone would like him. You hadn't thought of what you'd say. Now you wish you'd stayed quiet.",
            unlockable: "Say nothing.",
            success:
              "You don't say anything, and as people trail off wondering what your deal is, your mystique grows. Finally they show you the backroom secret poker game.",
          },
        ],
      },
      {
        label:
          "This illegal poker game consists of individuals as different from ordinary people as you are. This guy is dropping _zero_ tells.",
        options: [
          {
            skill: () => party2,
            locked: "Lose money.",
            failure:
              "You can bet big against this guy -- and he calls you -- or you can call him -- and he raises you -- and you're never ever up.",
            unlockable: "Make up an insulting nickname.",
            success:
              "MR. GOOFY GLASSES, you call him. At first he looks down his nose like you belong on his shoe. Then the others join in. He runs in disgrace.",
          },
        ],
      },
    ],
  },
  {
    // library
    stats: { primary: "INT", secondary: "CHA" },
    roomLabels: {
      hall: "Library",
      backroom: "Special Collections",
      closet: "Hall of Desire",
    },
    thrall: () => thrallStare,
    thrallItem: () => thrallLore,
    checks: [
      {
        label:
          "Special Collections. This guy is not just a librarian -- he's a vampire, too -- which he makes no effort to hide.",
        options: [
          {
            skill: () => party1,
            locked: "Quietly do nothing.",
            failure:
              "He airily crosses the room as if his feet aren't touching the ground. Your silence is subsumed into his and you form a non-library-disturbing collective.",
            unlockable: "Be super loud.",
            success:
              'You summon MDMA energy into your immortal coil and before you\'ve opened your mouth he resigns to you. "Here are the books." He fades.',
          },
          {
            skill: () => charm0,
            locked: "Gawk at him.",
            failure:
              "He's so cool. Every day you remember you're a vampire and vampires are so, so, cool.",
            unlockable: "Say he's cool.",
            success:
              "Looks like he gets that a lot. He's not fazed. \"I'm going to let you back here, because you need this,\" he says.",
          },
        ],
      },
      {
        label:
          "The librarian took a big risk letting you in back here. He's obviously deciding whether or not he's made a mistake.",
        options: [
          {
            skill: () => lore2,
            locked: "Look at the books.",
            failure:
              "DISCOURSE ON THE LOTUS SUTRA, you read, from the spine of a random book. He's listening but pretending not to be paying attention.",
            unlockable: "Prove you read something.",
            success:
              '"Fruit bats," you say. "From the story. They\'re not actually bats, they\'re --"\n"Metaphorical," he agrees. "But for what?"',
          },
        ],
      },
    ],
  },
];
